
# BEACH

<!-- badges: start -->
<!-- badges: end -->

Biometric Exploratory Analysis Creation House (BEACH) is a shiny app that provides automation platform for users.


## Installation

You can install and load some packages before running BEACH like so:

``` r
dep.packages <- c("shiny", "DT", "haven", "xtable", "plyr", "sas7bdat", "WriteXLS");
na.packages <- dep.packages[!dep.packages %in% installed.packages()];
if (length(na.packages)>0) install.packages(na.packages);

devtools::install_github('DanniYuGithub/RTF2', force=TRUE);
if(!"sas7bdat.parso" %in% installed.packages()) devtools::install_github('BioStatMatt/sas7bdat.parso', force=TRUE);

if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager");
BiocManager::install("Heatplus");
BiocManager::install("multtest");

  library(animation)

  library(beanplot)

  library(corrplot)  #create heatmap for correlation matrix#

  library(DT) #for render table

  library(grid)
  library(gridExtra)

  library(haven) #for loading SAS datasets
  library(Heatplus)  #creating customized heatmaps with covariate profile#
  library(Hmisc)

  library(lattice)
  library(latticeExtra)
  library(lmtest)

  library(multtest)

  library(plotrix)   #use 2D-heatmap#
  library(plyr)

  library(sas7bdat.parso)
  library(shiny)
  library(sp)
  library(survival)  #time to event analysis#

  library(vioplot)
  
  library(xtable)
  library(readxl)

  library(WriteXLS)

```

## Example

This is a basic example which shows you how to solve a common problem:

``` r
library(BEACH)
## basic example code
run_app()
```

