# Run the BEACH Shiny App
#
# @export
run_app <- function() {
  app_dir <- system.file("app", package = "BEACHpkg")
  if (app_dir == "") stop("App not found")
  shiny::runApp(app_dir, display.mode = "normal")
}